/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2013 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.01 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_01.txt                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Santiago Lizardo <santiagolizardo@php.net>                   |
  +----------------------------------------------------------------------+
*/

#include "php_sdl_cdrom.h"

#include <SDL.h>

/* {{{ proto int sdl_cdindrive(int status)

 */
// #define CD_INDRIVE(status)	((int)status > 0)
PHP_FUNCTION(sdl_cd_indrive)
{
	zval **status_arg;
	int status;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", status_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	convert_to_long_ex(status_arg);
	status = Z_LVAL_PP(status_arg);

	RETURN_LONG(CD_INDRIVE(status));
}
/* }}} */

/* {{{ proto void sdl_frames_to_msf(int f, int &M, int &S, int &F)

 */
PHP_FUNCTION(sdl_frames_to_msf)
{
	long frame;
	int m, s, f;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &frame) == FAILURE)
		return;
	
	FRAMES_TO_MSF(frame, &m, &s, &f);

	array_init(return_value);
	add_next_index_long(return_value, m);
	add_next_index_long(return_value, s);
	add_next_index_long(return_value, f);
}
/* }}} */

/* {{{ proto int sdl_msf_to_frames(int M, int S, int F)

 */
// #define MSF_TO_FRAMES(M, S, F)	((M)*60*CD_FPS+(S)*CD_FPS+(F))
PHP_FUNCTION(sdl_msf_to_frames)
{
	zval **M_arg, **S_arg, **F_arg;
	int M, S, F;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zzz", M_arg, S_arg, F_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	convert_to_long_ex(M_arg);
	convert_to_long_ex(S_arg);
	convert_to_long_ex(F_arg);

	M = Z_LVAL_PP(M_arg);
	S = Z_LVAL_PP(S_arg);
	F = Z_LVAL_PP(F_arg);

	RETURN_LONG(MSF_TO_FRAMES(M, S, F));
}
/* }}} */

/* {{{ proto int sdl_cdnumdrives(void)

 */
// int SDL_CDNumDrives(void);
PHP_FUNCTION(sdl_cdnumdrives)
{
	if (zend_parse_parameters_none() == FAILURE)
		return;

	RETURN_LONG(SDL_CDNumDrives());
}
/* }}} */

/* {{{ proto int sdl_cdname(int drive)

 */
// const char *SDL_CDName(int drive);
PHP_FUNCTION(sdl_cdname)
{
	zval **drive_arg;
	int drive;
	char *buf;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", drive_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	convert_to_long_ex(drive_arg);
	drive = Z_LVAL_PP(drive_arg);

	buf = (char *)SDL_CDName(drive);

	RETURN_STRING(estrndup(buf,strlen(buf)),0);
}
/* }}} */

/* {{{ proto array sdl_cdopen(int drive)

 */
// SDL_CD *SDL_CDOpen(int drive);
PHP_FUNCTION(sdl_cdopen)
{
	zval **drive_arg, *one_track, *track_array;
	int handle, drive, cur_track;
	SDL_CD *cdrom = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", drive_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	convert_to_long_ex(drive_arg);
	drive = Z_LVAL_PP(drive_arg);

	cdrom = SDL_CDOpen(drive);
	if (!cdrom) {
		
			php_error(E_WARNING, "%s() couldn't open drive: %s",
					  get_active_function_name(TSRMLS_C), SDL_GetError());
		
		RETURN_FALSE;
	}

	// SDL_CDOpen does not return track informations, but let's initialize the return array
	// first.  Track informations will get populated after SDL_CDStatus() call.
	MAKE_STD_ZVAL(track_array);
	array_init(track_array);
	for(cur_track=0; cur_track <= SDL_MAX_TRACKS; cur_track++) {
		MAKE_STD_ZVAL(one_track);
		array_init(one_track);
		add_assoc_long(one_track, "id", cdrom->track[cur_track].id);
		add_assoc_long(one_track, "type", cdrom->track[cur_track].type);
		add_assoc_long(one_track, "unused", cdrom->track[cur_track].unused);
		add_assoc_long(one_track, "length", cdrom->track[cur_track].length);
		add_assoc_long(one_track, "offset", cdrom->track[cur_track].offset);
		add_next_index_zval(track_array, one_track);
	}

	handle = zend_list_insert(cdrom, le_cdrom TSRMLS_CC);

	array_init(return_value);
	add_assoc_resource(return_value, "handle", handle);
	zend_list_addref(handle);

	add_assoc_long(return_value, "id", cdrom->id);
	add_assoc_long(return_value, "status", cdrom->status);
	add_assoc_long(return_value, "numtracks", cdrom->numtracks);
	add_assoc_long(return_value, "cur_track", cdrom->cur_track);
	add_assoc_long(return_value, "cur_frame", cdrom->cur_frame);
	add_assoc_zval(return_value, "track", track_array);

}
/* }}} */

/* {{{ proto int sdl_cdstatus(array &cdrom)

 */
// CDstatus SDL_CDStatus(SDL_CD *cdrom);
PHP_FUNCTION(sdl_cdstatus)
{
	zval **cdrom_arg, **handle_resource, **track_array, **one_track;
	int cdstatus, cur_track;
	SDL_CD *cdrom = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", cdrom_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	/* check for parameter being passed by reference */
	if (!PZVAL_IS_REF(*cdrom_arg)) {
		php_error(E_WARNING, "%s() parameter cdrom needs to be passed by reference",
				  get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	convert_to_array_ex(cdrom_arg);

	if (zend_hash_find(Z_ARRVAL_PP(cdrom_arg), "handle", sizeof("handle"), (void **)&handle_resource) == FAILURE) {
		php_error(E_WARNING, "%s() unable to find cdrom['handle'] resource",
				  get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	if (zend_hash_find(Z_ARRVAL_PP(cdrom_arg), "track", sizeof("track"), (void **)&track_array) == FAILURE) {
		php_error(E_WARNING, "%s() unable to find cdrom['track'] array",
				  get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	ZEND_FETCH_RESOURCE(cdrom, SDL_CD *, handle_resource, -1, le_cdrom_name, le_cdrom);
	cdstatus = SDL_CDStatus(cdrom);

	// Update array &cdrom according to contents in SDL_CD *cdrom strucutre
	add_assoc_long((*cdrom_arg), "status", cdrom->status);
	add_assoc_long((*cdrom_arg), "numtracks", cdrom->numtracks);
	add_assoc_long((*cdrom_arg), "cur_track", cdrom->cur_track);
	add_assoc_long((*cdrom_arg), "cur_frame", cdrom->cur_frame);

	for(cur_track=0;cur_track <= cdrom->numtracks; cur_track++) {
		if (zend_hash_index_find(Z_ARRVAL_PP(track_array), cur_track, (void **)&one_track) == FAILURE) {
			php_error(E_WARNING, "%s() unable to find cdrom['track'][%d] array", cur_track,
				  get_active_function_name(TSRMLS_C));
		}

		add_assoc_long((*one_track), "id", cdrom->track[cur_track].id);
		add_assoc_long((*one_track), "type", cdrom->track[cur_track].type);
		add_assoc_long((*one_track), "unused", cdrom->track[cur_track].unused);
		add_assoc_long((*one_track), "length", cdrom->track[cur_track].length);
		add_assoc_long((*one_track), "offset", cdrom->track[cur_track].offset);
	}

	RETURN_LONG(cdstatus);
}
/* }}} */


#define GET_CDROM_RESOURCE(cdrom)	{ \
										switch (Z_TYPE_PP(cdrom_arg)) { \
											case IS_ARRAY: \
												if (zend_hash_find(Z_ARRVAL_PP(cdrom_arg), "handle", sizeof("handle"), (void**)&handle_resource) == FAILURE) { \
													php_error(E_WARNING, "%s() unable to find cdrom['handle'] resource", \
															  get_active_function_name(TSRMLS_C)); \
													RETURN_FALSE; \
												} \
											case IS_RESOURCE: \
												ZEND_FETCH_RESOURCE(cdrom, SDL_CD *, handle_resource, -1, le_cdrom_name, le_cdrom); \
												break; \
											default: \
												php_error(E_WARNING, "%s() argument 1 expected to be CDROM resource or array containing CDROM resource; %s given", \
														  get_active_function_name(TSRMLS_C), zend_zval_type_name(*cdrom_arg)); \
												RETURN_FALSE; \
										} \
									}


/* {{{ proto int sdl_cdplay(array cdrom, int start, int length)

 */
// int SDL_CDPlay(SDL_CD *cdrom, int start, int length);
PHP_FUNCTION(sdl_cdplay)
{
	zval **cdrom_arg, **start_arg, **length_arg, **handle_resource;
	int start, length;
	SDL_CD *cdrom;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zzz", cdrom_arg, start_arg, length_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	GET_CDROM_RESOURCE(cdrom);

	convert_to_long_ex(start_arg);
	convert_to_long_ex(length_arg);

	start = Z_LVAL_PP(start_arg);
	length = Z_LVAL_PP(length_arg);

	RETURN_LONG(SDL_CDPlay(cdrom, start, length));
}
/* }}} */

/* {{{ proto int sdl_cdplaytracks(array cdrom, int start_track, int start_frame, int ntracks, int nframes)

 */
// int SDL_CDPlayTracks(SDL_CD *cdrom, int start_track, int start_frame, int ntracks, int nframes));
PHP_FUNCTION(sdl_cdplaytracks)
{
	zval **cdrom_arg, **start_track_arg, **start_frame_arg, **ntracks_arg, **nframes_arg, **handle_resource;
	int start_track, start_frame, ntracks, nframes, ret;
	SDL_CD *cdrom;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zzzzz", cdrom_arg, start_track_arg, start_frame_arg, ntracks_arg, nframes_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	GET_CDROM_RESOURCE(cdrom);

	convert_to_long_ex(start_track_arg);
	convert_to_long_ex(start_frame_arg);
	convert_to_long_ex(ntracks_arg);
	convert_to_long_ex(nframes_arg);

	start_track = Z_LVAL_PP(start_track_arg);
	start_frame = Z_LVAL_PP(start_frame_arg);
	ntracks = Z_LVAL_PP(ntracks_arg);
	nframes = Z_LVAL_PP(nframes_arg);

	if (-1 == (ret = SDL_CDPlayTracks(cdrom, start_track, start_frame, ntracks, nframes))) {
		RETURN_FALSE;
	} else {
		RETURN_LONG(ret);
	}
}
/* }}} */

/* {{{ proto int sdl_cdpause(array cdrom)

 */
// int SDL_CDPause(SDL_CD *cdrom);
PHP_FUNCTION(sdl_cdpause)
{
	zval **cdrom_arg, **handle_resource;
	SDL_CD *cdrom;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", cdrom_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	GET_CDROM_RESOURCE(cdrom);

	RETURN_LONG(SDL_CDPause(cdrom));
}
/* }}} */

/* {{{ proto int sdl_cdresume(array cdrom)

 */
// int SDL_CDResume(SDL_CD *cdrom);
PHP_FUNCTION(sdl_cdresume)
{
	zval **cdrom_arg, **handle_resource;
	SDL_CD *cdrom;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", cdrom_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	GET_CDROM_RESOURCE(cdrom);

	RETURN_LONG(SDL_CDResume(cdrom));
}
/* }}} */

/* {{{ proto int sdl_cdstop(array cdrom)

 */
// int SDL_CDStop(SDL_CD *cdrom);
PHP_FUNCTION(sdl_cdstop)
{
	zval **cdrom_arg, **handle_resource;
	SDL_CD *cdrom;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", cdrom_arg) == FAILURE) { 
		WRONG_PARAM_COUNT;
	}

	GET_CDROM_RESOURCE(cdrom);

	RETURN_LONG(SDL_CDStop(cdrom));
}
/* }}} */

/* {{{ proto int sdl_cdeject(array cdrom)

 */
// int SDL_CDEject(SDL_CD *cdrom);
PHP_FUNCTION(sdl_cdeject)
{
	zval **cdrom_arg, **handle_resource;
	SDL_CD *cdrom;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", cdrom_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	GET_CDROM_RESOURCE(cdrom);

	RETURN_LONG(SDL_CDEject(cdrom));
}
/* }}} */

/* {{{ proto void sdl_cdclose(array cdrom)

 */
// void SDL_CDClose(SDL_CD *cdrom);
PHP_FUNCTION(sdl_cdclose)
{
	zval **cdrom_arg, **handle_resource;
	int handle;
	SDL_CD *cdrom = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", cdrom_arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	GET_CDROM_RESOURCE(cdrom);
	handle = Z_LVAL_PP(handle_resource);
	zend_list_delete(handle);

	SDL_CDClose(cdrom);
}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
